package com.assignment;

import com.assignment.util.Student;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

public class Java8FilterExample {

    //filter() method in java8 programming is used to filter the elements on the basis of a given predicate.
    //syntax - Stream<T> filter(Predicate<? super T> predicate)
    //where predicate parameter takes predicate reference as an argument (i.e. a lambda expression)

    //method #1
    private static void filterCollection() {
        final List<String> names = Arrays.asList("John", "Adam", "Patrick", "Melisandre", "Sansa", "Daenerys");

        names
                .stream()
                .filter(name -> name.length() > 5)  //filtering long names
                .forEach(System.out::println);  //printing the results
    }

    //method #2
    private static void filterStudentCollectionAndCollect() {
        final List<Student> students = Student.createStudents();

        final List<Student> filteredStudents = students
                .stream()
                .filter(student -> student.getCgpa() > 7.0) //filtering students whose cgpa is greater than 7.0
                .collect(Collectors.toList());
        filteredStudents.forEach(student -> System.out.println(student.toString()));    //printing the results
    }

    //method #3
    private static void filterStudentCollectionWithMultipleCriteria() {
        final List<Student> students = Student.createStudents();

        final Student student = students
                .stream()
                .filter(stu -> stu.getCgpa() > 7.0) //filtering students whose cgpa is greater than 7.0
                .filter(stu1 -> stu1.getName().startsWith("Mary T."))   //filtering student whose name is Mary T.
                .findAny()
                .orElse(null);
        System.out.println(student.toString());
    }

    //main() method for execution
    public static void main(String[] args) {
        filterCollection();
        System.out.println("---------------------------");
        filterStudentCollectionAndCollect();
        System.out.println("---------------------------");
        filterStudentCollectionWithMultipleCriteria();
    }
}
